'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _detectors = require('babel-plugin-styled-components/lib/utils/detectors');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Validates if a css rule contains a self reference "&". It is valid if
 * `.someClass & {` or `& {`, but, in case of `.myClass { ... } .myClass + & {`
 * it would be false as the regex requires the first rule be based on a self
 * reference.
 * @param {string} css
 */
var startsWithSelfReference = function startsWithSelfReference(css) {
  return (/^(([^\{]([\w-_,:+\s]+))*\&\s*([^\{]([\w-_,:+\s]+))*)+[\{]/g.test(css.trim())
  );
};

var getCssNamespace = function getCssNamespace(state) {
  var _state$opts = state.opts,
      cssNamespace = _state$opts.cssNamespace,
      rawCssNamespace = _state$opts.rawCssNamespace;

  if (!cssNamespace && !rawCssNamespace) {
    return { cssNamespace: '&&', hasNamespaceSelfReference: true };
  }

  var isCssNamespaceSet = !!cssNamespace;
  var wrapperClass = isCssNamespaceSet ? `.${[].concat(cssNamespace).join(' .')} &` : `${[].concat(rawCssNamespace).join(', ')}`;
  var hasNamespaceSelfReference = startsWithSelfReference(`${wrapperClass} {`);
  return { cssNamespace: wrapperClass, hasNamespaceSelfReference };
};

var wrapCssDefinition = function wrapCssDefinition(cssNamespace, css, isSelfReferenced, shouldLeaveScopeOpen) {
  return isSelfReferenced ? `${cssNamespace} {${css}${shouldLeaveScopeOpen ? '' : '}'}` : `${cssNamespace} {&{${css}${shouldLeaveScopeOpen ? '' : '}}'}`;
};

var NO_STRIP = false;
var STRIP_END_CURLY_BRACKETS = true;

exports.default = function (path, state) {
  var _getCssNamespace = getCssNamespace(state),
      cssNamespace = _getCssNamespace.cssNamespace,
      hasNamespaceSelfReference = _getCssNamespace.hasNamespaceSelfReference;

  if ((0, _detectors.isStyled)(path.node.tag, state) && path.node.quasi.quasis[0].value.cooked && !path.node.quasi.quasis[0].value.cooked.startsWith(`\n${cssNamespace} {`) && (path.node.tag.property ? path.node.tag.property.name !== 'keyframes' : true)) {
    var _path$node = path.node,
        callee = _path$node.tag,
        _path$node$quasi = _path$node.quasi,
        quasis = _path$node$quasi.quasis,
        expressions = _path$node$quasi.expressions;


    var values = void 0;
    if (quasis.length === 1) {
      values = [t.templateElement({
        cooked: `\n${wrapCssDefinition(cssNamespace, quasis[0].value.cooked, hasNamespaceSelfReference || startsWithSelfReference(quasis[0].value.cooked), NO_STRIP)}\n`,
        raw: `\n${wrapCssDefinition(cssNamespace, quasis[0].value.raw, hasNamespaceSelfReference || startsWithSelfReference(quasis[0].value.raw), NO_STRIP)}\n`
      }, quasis[0].tail)];
    } else {
      var isCookedSelfReferenced = hasNamespaceSelfReference || startsWithSelfReference(quasis[0].value.cooked);
      var isRawSelfReferenced = hasNamespaceSelfReference || startsWithSelfReference(quasis[0].value.raw);

      var first = t.templateElement({
        cooked: `\n${wrapCssDefinition(cssNamespace, quasis[0].value.cooked, isCookedSelfReferenced, STRIP_END_CURLY_BRACKETS)}`,
        raw: `\n${wrapCssDefinition(cssNamespace, quasis[0].value.raw, isRawSelfReferenced, STRIP_END_CURLY_BRACKETS)}`
      }, quasis[0].tail);

      var last = t.templateElement({
        cooked: `${quasis[quasis.length - 1].value.cooked}${isCookedSelfReferenced ? '}' : '}}'}\n`,
        raw: `${quasis[quasis.length - 1].value.raw}${isRawSelfReferenced ? '}' : '}}'}\n`
      }, quasis[quasis.length - 1].tail);
      values = [first].concat(_toConsumableArray(quasis.slice(1, quasis.length - 1)), [last]);
    }

    path.replaceWith(t.taggedTemplateExpression(callee, t.templateLiteral(values, expressions)));
  }
};